@@ -1,5 +1,5 @@
 /*
-	SpamTestBuddy 0.93
+	SpamTestBuddy 0.94
 
 	Copyright (C) 2007 Jem E. Berkes
 	http://www.sysdesign.ca/
@@ -26,7 +26,7 @@
 #include <arpa/inet.h>
 #include "spamtestbuddy.h"
 
-#define VERSION		"0.93"
+#define VERSION		"0.94"
 
 /* #define VERBOSE if you want some useful debugging output to stderr */
 
@@ -34,13 +34,15 @@
 
 
 void process_headers();
-void output_test_results(const char* ipstr);
+int test_dns_problems(const char* ipstr);
+void test_file_ip(const char* ipstr);
 void test_header_substr(const char* linebuf);
 void test_header_yes(const char* linebuf);
 void test_header_float(const char* linebuf);
 void test_dnsbl(unsigned int* ip);
+void output_test_results(const char* ipstr);
+void show_matches(struct testentry* pos);
 int ip_is_local(const char* ipstr);
-int test_dns_problems(const char* ipstr);
 int load_configuration(FILE* config);
 int add_test(struct testentry** chain, const char* prefix, const char* testname, const char* param);
 void show_config();
@@ -52,6 +54,7 @@ float score = 0;
 float threshold = 0;				/* SpamThreshold */
 char local_hops[MAXLINEBUF] = "";		/* SkipReceived */
 struct testentry* testdnsproblems = NULL;	/* TestDnsProblems */
+struct testentry* testfileip = NULL;		/* TestFileIP */
 struct testentry* testheadersubstr = NULL;	/* TestHeaderSubstr */
 struct testentry* testheaderyes = NULL;		/* TestHeaderYes */
 struct testentry* testheaderfloat = NULL;	/* TestHeaderFloat */
@@ -123,6 +126,9 @@ void process_headers()
 				else
 					score -= SCOREUNIT;
 			}
+			/* TestFileIP */
+			if (got_ip)
+				test_file_ip(ipstr);	/* scores applied in function */
 			/* TestDNSBL */
 			if (got_ip)
 				test_dnsbl(ip);		/* scores applied in function */
@@ -163,40 +169,6 @@ void process_headers()
 }
 
 
-/*
-	Show results of all tests done (scores already applied)
-*/
-void output_test_results(const char* ipstr)
-{
-	struct testentry* pos;
-	const char* label;
-	if (score >= threshold)
-		label = "OVER";
-	else
-		label = "UNDER";
-	printf("X-SpamTestBuddy: %s score=%.4f threshold=%.4f ip=%s\n", label, score, threshold, ipstr);
-	printf("X-SpamTestBuddy-Tests: ver=" VERSION " ");
-	/* Show testnames that go along with all tests which matched */
-	if (testdnsproblems && testdnsproblems->match)
-		printf("%s ", testdnsproblems->testname);
-	for (pos=testheadersubstr; pos; pos=pos->next)
-	{
-		if (pos->match) printf("%s ", pos->testname);
-	}
-	for (pos=testheaderyes; pos; pos=pos->next)
-	{
-		if (pos->match) printf("%s ", pos->testname);
-	}
-	for (pos=testheaderfloat; pos; pos=pos->next)
-	{
-		if (pos->match) printf("%s ", pos->testname);
-	}
-	for (pos=testdnsbl; pos; pos=pos->next)
-	{
-		if (pos->match) printf("%s ", pos->testname);
-	}
-	puts("\n");
-}
 
 
 /*
@@ -283,9 +255,47 @@ void test_header_float(const char* linebuf)
 }
 
 
+/*
+	Test IP against files containing IP lists
+	The function is called once per email
+*/
+void test_file_ip(const char* ipstr)
+{
+	struct testentry* pos = testfileip;
+	while (pos)
+	{
+		FILE* localfile = fopen(pos->param, "r");
+		if (localfile)
+		{
+			char fileline[MAXLINEBUF];
+			while (!pos->match && fgets(fileline, sizeof(fileline), localfile))
+			{
+				char* end = strchr(fileline, '\r');
+				if (!end) end = strchr(fileline, '\n');
+				if (end) *end = '\0';
+				/* do exact comparison */
+				if (strcmp(ipstr, fileline) == 0)
+				{
+					pos->match = 1;
+					if (pos->adds)
+						score += SCOREUNIT;
+					else
+						score -= SCOREUNIT;
+				}
+			}
+			fclose(localfile);
+		}
+		else
+			perror(pos->param);	/* error to stderr if could not open file */
+		
+		pos = pos->next;
+	}
+}
+
+
 /*
 	Test all DNSBLs, updating scores
-	Unlike test_header's, this function is only called once (one IP to test per email)
+	The function is called once per amil
 */
 void test_dnsbl(unsigned int* ip)
 {
@@ -376,6 +386,38 @@ int ip_is_local(const char* ipstr)
 }
 
 
+/*
+	Show results of all tests done (scores already applied)
+*/
+void output_test_results(const char* ipstr)
+{
+	const char* label;
+	if (score >= threshold)
+		label = "OVER";
+	else
+		label = "UNDER";
+	printf("X-SpamTestBuddy: %s score=%.4f threshold=%.4f ip=%s\n", label, score, threshold, ipstr);
+	printf("X-SpamTestBuddy-Tests: ver=" VERSION " ");
+	/* Show testnames that go along with all tests which matched */
+	show_matches(testdnsproblems);
+	show_matches(testfileip);
+	show_matches(testheadersubstr);
+	show_matches(testheaderyes);
+	show_matches(testheaderfloat);
+	show_matches(testdnsbl);
+	puts("\n");
+}
+
+
+void show_matches(struct testentry* pos)
+{
+	for (; pos; pos=pos->next)
+	{
+		if (pos->match)
+			printf("%s ", pos->testname);
+	}	
+}
+
 
 /*
 	Load, parse the configuration file
@@ -403,6 +445,11 @@ int load_configuration(FILE* config)
 			if (add_test(&testdnsproblems, prefix, testname, ""))
 				return 1;
 		}
+		else if (sscanf(linebuf, "%[+-]TestFileIP %s %s", prefix, testname, param) == 3)
+		{
+			if (add_test(&testfileip, prefix, testname, param))
+				return 1;
+		}
 		else if (sscanf(linebuf, "%[+-]TestHeaderSubstr %s %[^\r\n]", prefix, testname, param) == 3)
 		{
 			if (add_test(&testheadersubstr, prefix, testname, param))
@@ -489,6 +536,8 @@ void show_config()
 	fprintf(stderr, "\nadd/sub\ttest\t\tname\tparam\n\n");
 	if (testdnsproblems)
 		fprintf(stderr, "%d\tTestDnsProblems\t%s\tN/A\n", testdnsproblems->adds, testdnsproblems->testname);
+	for (pos=testfileip; pos; pos=pos->next)
+		fprintf(stderr, "%d\tTestFileIP\t%s\t%s\n", pos->adds, pos->testname, pos->param);
 	for (pos=testheadersubstr; pos; pos=pos->next)
 		fprintf(stderr, "%d\tTestHeaderSubstr\t%s\t%s\n", pos->adds, pos->testname, pos->param);
 	for (pos=testheaderyes; pos; pos=pos->next)
